import json

from django.shortcuts import get_object_or_404,render
from django.db.models import F


# Create your views here.
from django.http import HttpResponse,HttpResponseRedirect,JsonResponse
from .models import Question,Choice
from django.template import loader
from django.http import Http404
from django.urls import reverse
import requests
from django.conf import settings


def index(request):
    latest_question_list = Question.objects.order_by("-pub_date")[:5]
    #template = loader.get_template("polls/index.html")#
    context = {
        "latest_question_list":latest_question_list,
        "latest_question_list":latest_question_list,
    }
    return render(request,"polls/index.html",context)

def detail(request, question_id):
    '''try:
        question = Question.objects.get(pk = question_id)
    except Question.DoesNotExist:
        raise Http404("Question does not exits")'''
    question = get_object_or_404(Question,pk = question_id)
    return render(request,"polls/detail.html",{"question":question})


def results(request, question_id):
    question = get_object_or_404(Question,pk=question_id)
    return render(request,"polls/results.html",{"question":question})


def vote(request, question_id):
    question = get_object_or_404(Question,pk=question_id)
    try:
        selected_choice = question.choice_set.get(pk=request.POST["choice"])
    except(KeyError,Choice.DoesNotExist):
        return render(
            request,
            "polls/detail.html",
            {
                "question":question,
                "error_message":"You didn't select a choice.",
            }
        )
    else:
        selected_choice.votes = F("votes")+1
        selected_choice.save()
        return HttpResponseRedirect(reverse("polls:results",args=(question.id,)))
    

def trigger_pipeline(request):
    
    if request.method == "POST":
        url = settings.GITLAB_PIPELINE_TRIGGER_URL #gitlab接口请求地址
        ref = request.POST.get('ref') #需要选择打包的分支
        build_frontend_ref = request.POST.get('build_frontend_ref') #前端变量
        build_backend_ref = request.POST.get('build_backend_ref') #后端变量
        headers = {
            'PRIVATE-TOKEN': settings.GITLAB_TOKEN,
        }
        data = {
            'token': settings.GITLAB_TRIGGER_TOKEN,
            'ref': ref,
            'variables[BUILD_FRONTEND_REF]': build_frontend_ref,
            'variables[BUILD_BACKEND_REF]': build_backend_ref
        }
        response = requests.post(url, headers=headers, data=data)
        if response.status_code == 201:
            print("trigger success")
            messages = "trigger success"
            url = reverse("polls:trigger_page") + f'?messages={messages}'
            return HttpResponseRedirect(url)
        else:
            print("trigger failed")
            messages = "trigger failed"
            url = reverse("polls:trigger_page") + f'?messages={messages}'
            return HttpResponseRedirect(url)
    else:
        print("wrong method")
        messages = "wrong method"
        url = reverse("polls:trigger_page") + f'?messages={messages}'
        
        return HttpResponseRedirect(url)

def trigger_page(request):
    message = request.GET.get('messages')
    if message == None:
        message = 'not ready'
    print(message)
    context = {
        "pipelines":pipeline_list(),
        "messages":message
    }
    #print(context["messages"])
    return render(request, 'polls/trigger_pipeline.html',context)

def pipeline_list():
    url = f'https://gitlab.syc-cms.com/api/v4/projects/{settings.GITLAB_PROJECT_ID}/pipelines'
    headers = {
        'PRIVATE-TOKEN':settings.GITLAB_TOKEN,
    }
    response = requests.get(url,headers=headers)
    response_dict = json.loads(response.text)

    if "error" in response_dict:
        print("in pipeline_list() method get_pipelines_response:" + response.text)
    else:
        all_pipelines = response.json()
        pipelines = [pipeline for pipeline in all_pipelines if pipeline['source']=='trigger']
        return pipelines

    '''context = {
        "pipelines":pipelines,
    }'''

